@@ -374,7 +374,13 @@ typedef struct {
     unsigned int	(*ddc1Read)(ScrnInfoPtr);
     void (*DDC1SetSpeed)(ScrnInfoPtr, xf86ddcSpeed);
     Bool		(*i2cInit)(ScrnInfoPtr);
-    I2CBusPtr		I2C;
+    I2CBusPtr		DDC_Bus1;
+    I2CBusPtr		DDC_Bus2;
+    I2CBusPtr		Maven_Bus;
+    I2CDevPtr		Maven;
+    char		Maven_Version;
+    Bool		UseMaven;
+    Bool		UseMavenPM;
     Bool		FBDev;
     int			colorKey;
     int			videoKey;
@@ -414,8 +420,10 @@ typedef struct {
 #endif
     XF86VideoAdaptorPtr adaptor;
     Bool		DualHeadEnabled;
+    Bool		Crtc2IsTV;
     Bool		SecondCrtc;
     Bool                SecondOutput;
+
     GDevPtr		device;
     /* The hardware's real SrcOrg */
     int			realSrcOrg;
@@ -575,6 +583,8 @@ Bool MgaInitDma(ScrnInfoPtr pScrn, int prim_size);
 
 #endif
 
+Bool MGAMavenRead(ScrnInfoPtr pScrn, I2CByte reg, I2CByte *val);
+
 void MGACRTC2Set(ScrnInfoPtr pScrn, xMODEINFO *pModeInfo);
 void MGAEnableSecondOutPut(ScrnInfoPtr pScrn, xMODEINFO *pModeInfo);
 void MGACRTC2SetPitch(ScrnInfoPtr pSrcn, xMODEINFO *pModeInfo);
