@@ -113,7 +113,6 @@ static unsigned int neo2200Rop[16] = {
     0x0f0000     /* 0x1111 - GXset           */
 };
 
-
 Bool 
 Neo2200AccelInit(ScreenPtr pScreen)
 {
@@ -252,8 +251,7 @@ Neo2200AccelInit(ScreenPtr pScreen)
 	return FALSE;
     }
 
-
-    return(XAAInit(pScreen, infoPtr));
+    return (xaaSetupWrapper(pScreen, infoPtr, pScrn->depth, &nPtr->accelSync));
 }
 
 static void
