@@ -1,4 +1,4 @@
-/* $XdotOrg: xc/programs/Xserver/hw/xfree86/drivers/nv/nv_driver.c,v 1.1.4.2 2003/12/20 00:28:27 kaleb Exp $ */
+/* $XdotOrg: xc/programs/Xserver/hw/xfree86/drivers/nv/nv_driver.c,v 1.5 2004/08/16 09:13:14 ajax Exp $ */
 /* $XConsortium: nv_driver.c /main/3 1996/10/28 05:13:37 kaleb $ */
 /*
  * Copyright 1996-1997  David J. McKay
@@ -288,16 +288,16 @@ static const char *fbdevHWSymbols[] = {
     "fbdevHWGetVidmem",
 
     /* colormap */
-    "fbdevHWLoadPalette",
+    "fbdevHWLoadPaletteWeak",
 
     /* ScrnInfo hooks */
-    "fbdevHWAdjustFrame",
+    "fbdevHWAdjustFrameWeak",
     "fbdevHWEnterVT",
-    "fbdevHWLeaveVT",
+    "fbdevHWLeaveVTWeak",
     "fbdevHWModeInit",
     "fbdevHWSave",
-    "fbdevHWSwitchMode",
-    "fbdevHWValidMode",
+    "fbdevHWSwitchModeWeak",
+    "fbdevHWValidModeWeak",
 
     "fbdevHWMapMMIO",
     "fbdevHWMapVidmem",
@@ -1053,11 +1053,11 @@ NVPreInit(ScrnInfoPtr pScrn, int flags)
 	    xf86FreeInt10(pNv->pInt);
 	    return FALSE;
 	}
-	pScrn->SwitchMode    = fbdevHWSwitchMode;
-	pScrn->AdjustFrame   = fbdevHWAdjustFrame;
+	pScrn->SwitchMode    = fbdevHWSwitchModeWeak();
+	pScrn->AdjustFrame   = fbdevHWAdjustFrameWeak();
 	pScrn->EnterVT       = NVEnterVTFBDev;
-	pScrn->LeaveVT       = fbdevHWLeaveVT;
-	pScrn->ValidMode     = fbdevHWValidMode;
+	pScrn->LeaveVT       = fbdevHWLeaveVTWeak();
+	pScrn->ValidMode     = fbdevHWValidModeWeak();
     }
     pNv->Rotate = 0;
     if ((s = xf86GetOptValString(pNv->Options, OPTION_ROTATE))) {
@@ -1765,7 +1765,7 @@ NVScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
     /* Initialize colormap layer.  
 	Must follow initialization of the default colormap */
     if(!xf86HandleColormaps(pScreen, 256, 8,
-	(pNv->FBDev ? fbdevHWLoadPalette : NVDACLoadPalette), 
+	(pNv->FBDev ? fbdevHWLoadPaletteWeak() : NVDACLoadPalette), 
 	NULL, CMAP_RELOAD_ON_MODE_SWITCH | CMAP_PALETTED_TRUECOLOR))
 	return FALSE;
 
