@@ -36,7 +36,6 @@
 #ifndef _WIN_H_
 #define _WIN_H_
 
-
 #ifndef NO
 #define NO					0
 #endif
@@ -44,17 +43,10 @@
 #define YES					1
 #endif
 
-/*
- * Build toggles for experimental features
- */
-#define WIN_NATIVE_GDI_SUPPORT			YES
-#define WIN_LAYER_SUPPORT			NO
-#define WIN_NEW_KEYBOARD_SUPPORT		NO
-#define WIN_EMULATE_PSEUDO_SUPPORT		YES
-#define WIN_UPDATE_STATS			NO
-
 /* Turn debug messages on or off */
+#ifndef CYGDEBUG
 #define CYGDEBUG				NO
+#endif
 
 #define NEED_EVENTS
 
@@ -68,7 +60,9 @@
 #define WIN_DEFAULT_WIN_KILL			TRUE
 #define WIN_DEFAULT_UNIX_KILL			FALSE
 #define WIN_DEFAULT_CLIP_UPDATES_NBOXES		0
+#ifdef XWIN_EMULATEPSEUDO
 #define WIN_DEFAULT_EMULATE_PSEUDO		FALSE
+#endif
 #define WIN_DEFAULT_USER_GAVE_HEIGHT_AND_WIDTH	FALSE
 
 #define WIN_DIB_MAXIMUM_SIZE	0x08000000 /* 16 MB on Windows 95, 98, Me */
@@ -107,8 +101,12 @@
 #define WIN_SERVER_SHADOW_GDI	0x1L	/* 1 */
 #define WIN_SERVER_SHADOW_DD	0x2L	/* 2 */
 #define WIN_SERVER_SHADOW_DDNL	0x4L	/* 4 */
+#ifdef XWIN_PRIMARYFB
 #define WIN_SERVER_PRIMARY_DD	0x8L	/* 8 */
-#define WIN_SERVER_NATIVE_GDI	0x10L	/* 16 */
+#endif
+#ifdef XWIN_NATIVEGDI
+# define WIN_SERVER_NATIVE_GDI	0x10L	/* 16 */
+#endif
 
 #define AltMapIndex		Mod1MapIndex
 #define NumLockMapIndex		Mod2MapIndex
@@ -133,8 +131,6 @@
 #include <errno.h>
 #include <pthread.h>
 
-#include <X11/XWDFile.h>
-
 #ifdef HAS_MMAP
 #include <sys/mman.h>
 #ifndef MAP_FILE
@@ -142,10 +138,10 @@
 #endif /* MAP_FILE */
 #endif /* HAS_MMAP */
 
-#include "X.h"
-#include "Xproto.h"
-#include "Xos.h"
-#include "Xprotostr.h"
+#include "X11/X.h"
+#include "X11/Xproto.h"
+#include "X11/Xos.h"
+#include "X11/Xprotostr.h"
 #include "scrnintstr.h"
 #include "pixmapstr.h"
 #include "pixmap.h"
@@ -158,20 +154,19 @@
 #include "windowstr.h"
 #include "mi.h"
 #include "micmap.h"
-#include "migc.h"
 #include "mifillarc.h"
 #include "mifpoly.h"
 #include "mibstore.h"
 #include "input.h"
 #include "mipointer.h"
-#include "keysym.h"
+#include "X11/keysym.h"
 #include "mibstore.h"
 #include "micoord.h"
 #include "dix.h"
 #include "miline.h"
 #include "shadow.h"
 #include "fb.h"
-#include "layer.h"
+#include "rootless.h"
 
 #ifdef RENDER
 #include "mipict.h"
@@ -186,6 +181,7 @@
  * Windows headers
  */
 #include "winms.h"
+#include "./winresource.h"
 
 
 /*
@@ -197,18 +193,9 @@
 #define WM_GIVEUP		(WM_USER + 1002)
 
 
-/*
- * Multi-Window Window Manager header
- */
-
+/* Local includes */
 #include "winwindow.h"
-
-
-/* Cygwin's winuser.h does not define VK_KANA as of 28Mar2001 */
-/* NOTE: Cygwin's winuser.h was fixed shortly after 28Mar2001. */
-#ifndef VK_KANA
-#define VK_KANA 15
-#endif
+#include "winmsg.h"
 
 
 /*
@@ -254,9 +241,27 @@ if (fDebugProcMsg) \
 #define DEBUGPROC_MSG
 #endif
 
+#define PROFILEPOINT(point,thresh)\
+{\
+static unsigned int PROFPT##point = 0;\
+if (++PROFPT##point % thresh == 0)\
+ErrorF (#point ": PROFILEPOINT hit %u times\n", PROFPT##point);\
+}
+
+
 /* We use xor this macro for detecting toggle key state changes */
 #define WIN_XOR(a,b) ((!(a) && (b)) || ((a) && !(b)))
 
+#define DEFINE_ATOM_HELPER(func,atom_name)			\
+static Atom func (void) {					\
+    static int generation;					\
+    static Atom atom;						\
+    if (generation != serverGeneration) {			\
+	generation = serverGeneration;				\
+	atom = MakeAtom (atom_name, strlen (atom_name), TRUE);	\
+    }								\
+    return atom;						\
+}
 
 /*
  * Typedefs for engine dependent function pointers
@@ -299,6 +304,12 @@ typedef Bool (*winCreatePrimarySurfaceProcPtr)(ScreenPtr);
 
 typedef Bool (*winReleasePrimarySurfaceProcPtr)(ScreenPtr);
 
+typedef Bool (*winFinishCreateWindowsWindowProcPtr)(WindowPtr pWin);
+
+
+/* Typedef for DIX wrapper functions */
+typedef int (*winDispatchProcPtr) (ClientPtr);
+
 
 /*
  * GC (graphics context) privates
@@ -337,19 +348,20 @@ typedef struct
   PALETTEENTRY		peColors[WIN_NUM_PALETTE_ENTRIES];
 } winPrivCmapRec, *winPrivCmapPtr;
 
-
-#if WIN_NEW_KEYBOARD_SUPPORT
 /*
- * Keyboard event structure
- */
+ * Windows Cursor handling.
+ */ 
+  
+typedef struct {
+  /* from GetSystemMetrics */
+  int sm_cx;
+  int sm_cy;
 
-typedef struct
-{
-  DWORD			dwXKeycodes[WIN_MAX_KEYS_PER_KEY];
-  DWORD			dwReleaseModifiers;
-} winKeyEventsRec, *winKeyEventsPtr;
-
-#endif /* WIN_NEW_KEYBOARD_SUPPORT */
+  BOOL visible;
+  HCURSOR handle;
+  QueryBestSizeProcPtr QueryBestSize;
+  miPointerSpriteFuncPtr spriteFuncs;
+} winCursorRec;
 
 /*
  * Screen information structure that we need before privates are available
@@ -387,18 +399,22 @@ typedef struct
   DWORD			dwDepth;
   DWORD			dwRefreshRate;
   char			*pfb;
-  XWDColor		*pXWDCmap;
-  XWDFileHeader		*pXWDHeader;
   DWORD			dwEngine;
   DWORD			dwEnginePreferred;
   DWORD			dwClipUpdatesNBoxes;
+#ifdef XWIN_EMULATEPSEUDO
   Bool			fEmulatePseudo;
+#endif
   Bool			fFullScreen;
   Bool			fDecoration;
+#ifdef XWIN_MULTIWINDOWEXTWM
+  Bool			fMWExtWM;
+#endif
   Bool			fRootless;
+#ifdef XWIN_MULTIWINDOW
   Bool			fMultiWindow;
+#endif
   Bool                  fMultipleMonitors;
-  Bool			fClipboard;
   Bool			fLessPointer;
   Bool			fScrollbars;
   Bool			fNoTrayIcon;
@@ -428,6 +444,8 @@ typedef struct _winPrivScreenRec
 
   int			iDeltaZ;
 
+  int			iConnectedClients;
+
   CloseScreenProcPtr	CloseScreen;
 
   DWORD			dwRedMask;
@@ -440,26 +458,11 @@ typedef struct _winPrivScreenRec
   /* Handle to icons that must be freed */
   HICON			hiconNotifyIcon;
 
-  /* Clipboard support */
-  pthread_t		ptClipboardProc;
-
-#if 0
-  HWND			hwndNextViewer;
-  void			*display;
-  int			window;
-#endif
-
   /* Last width, height, and depth of the Windows display */
   DWORD			dwLastWindowsWidth;
   DWORD			dwLastWindowsHeight;
   DWORD			dwLastWindowsBitsPixel;
 
-  /* Layer support */
-#if WIN_LAYER_SUPPORT
-  DWORD			dwLayerKind;
-  LayerPtr		pLayer;
-#endif
-
   /* Palette management */
   ColormapPtr		pcmapInstalled;
 
@@ -498,17 +501,29 @@ typedef struct _winPrivScreenRec
   /* Privates used by both shadow fb DirectDraw servers */
   LPDIRECTDRAWCLIPPER	pddcPrimary;
 
-  /* Privates used by multi-window server */
+#ifdef XWIN_MULTIWINDOWEXTWM
+  /* Privates used by multi-window external window manager */
+  RootlessFrameID	widTop;
+  Bool			fRestacking;
+#endif
+
+#ifdef XWIN_MULTIWINDOW
+  /* Privates used by multi-window */
   pthread_t		ptWMProc;
   pthread_t		ptXMsgProc;
   void			*pWMInfo;
-  Bool                  fWindowOrderChanged;
-  Bool                  fRestacking;
-  Bool			fRootWindowShown;
+#endif
 
+#if defined(XWIN_MULTIWINDOW) || defined(XWIN_MULTIWINDOWEXTWM)
+  /* Privates used by both multi-window and rootless */
+  Bool			fRootWindowShown;
+#endif
+
+#if defined(XWIN_CLIPBOARD) || defined(XWIN_MULTIWINDOW)
   /* Privates used for any module running in a seperate thread */
   pthread_mutex_t	pmServerStarted;
   Bool			fServerStarted;
+#endif
   
   /* Engine specific functions */
   winAllocateFBProcPtr			pwinAllocateFB;
@@ -530,6 +545,11 @@ typedef struct _winPrivScreenRec
   winCreatePrimarySurfaceProcPtr	pwinCreatePrimarySurface;
   winReleasePrimarySurfaceProcPtr	pwinReleasePrimarySurface;
 
+#ifdef XWIN_MULTIWINDOW
+  /* Window Procedures for MultiWindow mode */
+  winFinishCreateWindowsWindowProcPtr	pwinFinishCreateWindowsWindow;
+#endif
+
   /* Window Procedures for Rootless mode */
   CreateWindowProcPtr			CreateWindow;
   DestroyWindowProcPtr			DestroyWindow;
@@ -547,12 +567,35 @@ typedef struct _winPrivScreenRec
   ClipNotifyProcPtr			ClipNotify;
   RestackWindowProcPtr			RestackWindow;
   ReparentWindowProcPtr			ReparentWindow;
+  ResizeWindowProcPtr			ResizeWindow;
+  MoveWindowProcPtr			MoveWindow;
 #ifdef SHAPE
   SetShapeProcPtr			SetShape;
 #endif
+
+  winCursorRec                          cursor;
 } winPrivScreenRec;
 
 
+#ifdef XWIN_MULTIWINDOWEXTWM
+typedef struct {
+  RootlessWindowPtr	pFrame;
+  HWND			hWnd;
+  int			dwWidthBytes;
+  BITMAPINFOHEADER	*pbmihShadow;
+  HBITMAP		hbmpShadow;
+  HDC			hdcShadow;
+  HDC			hdcScreen;
+  BOOL			fResized;
+  BOOL			fRestackingNow;
+  BOOL			fClose;
+  BOOL			fMovingOrSizing;
+  BOOL			fDestroyed;//for debug
+  char			*pfb;
+} win32RootlessWindowRec, *win32RootlessWindowPtr;
+#endif
+
+
 typedef struct {
   pointer		value;
   XID			id;
@@ -576,8 +619,6 @@ extern unsigned long		g_ulServerGeneration;
 extern CARD32			g_c32LastInputEventTime;
 extern DWORD			g_dwEnginesSupported;
 extern HINSTANCE		g_hInstance;
-extern HWND			g_hDlgDepthChange;
-extern HWND			g_hDlgExit;
 extern int                      g_copyROP[];
 extern int                      g_patternROP[];
 extern const char *		g_pszQueryHost;
@@ -665,29 +706,6 @@ extern FARPROC			g_fpTrackMouseEvent;
 	winPrivWinPtr pWinPriv = winGetWindowPriv(pWin)
 
 
-/*
- * FIXME: Windows mouse wheel macro; should be in Cygwin w32api headers.
- * Has been fixed after May 05, 2001.  Remove this section after the
- * fixed headers are in distribution.
- */
-
-#ifndef GET_WHEEL_DELTA_WPARAM
-#define GET_WHEEL_DELTA_WPARAM(wparam)		((short)HIWORD (wparam))
-#endif /* GET_WHEEL_DELTA_WPARAM */
-
-
-/*
- * FIXME: Windows shell API defines.  Should be in w32api shellapi.h
- */
-
-#ifndef ABS_AUTOHIDE
-#define	ABS_AUTOHIDE		0x00000001
-#endif
-#ifndef ABS_ALWAYSONTOP
-#define	ABS_ALWAYSONTOP		0x00000002
-#endif
-
-
 /*
  * BEGIN DDX and DIX Function Prototypes
  */
@@ -701,12 +719,24 @@ Bool
 winAllocatePrivates (ScreenPtr pScreen);
 
 Bool
-winInitCmapPrivates (ColormapPtr pCmap);
+winInitCmapPrivates (ColormapPtr pCmap, int index);
 
 Bool
 winAllocateCmapPrivates (ColormapPtr pCmap);
 
 
+/*
+ * winauth.c
+ */
+
+#if defined(XWIN_CLIPBOARD) || defined(XWIN_MULTIWINDOW)
+# if defined(XCSECURITY)  
+Bool
+winGenerateAuthorization (void);
+# endif
+#endif
+
+
 /*
  * winblock.c
  */
@@ -718,72 +748,39 @@ winBlockHandler (int nScreen,
 		 pointer pReadMask);
 
 
+#ifdef XWIN_NATIVEGDI
 /*
  * winclip.c
  */
 
 RegionPtr
 winPixmapToRegionNativeGDI (PixmapPtr pPix);
+#endif
 
 
+#ifdef XWIN_CLIPBOARD
 /*
  * winclipboardinit.c
  */
 
 Bool
-winInitClipboard (pthread_t *ptClipboardProc,
-		  pthread_mutex_t *ppmServerStarted,
-		  DWORD dwScreen);
-
-/*
- * winclipboardthread.c
- */
+winInitClipboard (void);
 
 void
-winDeinitClipboard ();
+winFixClipboardChain (void);
+#endif
 
 
 /*
  * wincmap.c
  */
 
-int
-winListInstalledColormaps (ScreenPtr pScreen, Colormap *pmaps);
-
 void
-winStoreColors (ColormapPtr pmap, int ndef, xColorItem *pdefs);
-
-void
-winInstallColormap (ColormapPtr pmap);
-
-void
-winUninstallColormap (ColormapPtr pmap);
-
-void
-winResolveColor (unsigned short *pred,
-		 unsigned short *pgreen,
-		 unsigned short *pblue,
-		 VisualPtr	pVisual);
-
-Bool
-winCreateColormap (ColormapPtr pmap);
-
-void
-winDestroyColormap (ColormapPtr pmap);
-
-int
-winExpandDirectColors (ColormapPtr pmap, int ndef,
-		       xColorItem *indefs, xColorItem *outdefs);
+winSetColormapFunctions (ScreenPtr pScreen);
 
 Bool
 winCreateDefColormap (ScreenPtr pScreen);
 
-Bool
-winSetVisualTypesNativeGDI (int nDepth, int nVisuals, int nBitsPerRGB);
-
-void
-winClearVisualTypes (void);
-
 
 /*
  * wincreatewnd.c
@@ -796,20 +793,6 @@ Bool
 winCreateBoundingWindowWindowed (ScreenPtr pScreen);
 
 
-/*
- * wincursor.c
- */
-
-void
-winPointerWarpCursor (ScreenPtr pScreen, int x, int y);
-
-Bool
-winCursorOffScreen (ScreenPtr *ppScreen, int *x, int *y);
-
-void
-winCrossScreen (ScreenPtr pScreen, Bool fEntering);
-
-
 /*
  * windialogs.c
  */
@@ -817,33 +800,41 @@ winCrossScreen (ScreenPtr pScreen, Bool fEntering);
 void
 winDisplayExitDialog (winPrivScreenPtr pScreenPriv);
 
-
 void
 winDisplayDepthChangeDialog (winPrivScreenPtr pScreenPriv);
 
+void
+winDisplayAboutDialog (winPrivScreenPtr pScreenPriv);
+
 
 /*
  * winengine.c
  */
 
 void
-winDetectSupportedEngines ();
+winDetectSupportedEngines (void);
 
 Bool
 winSetEngine (ScreenPtr pScreen);
 
 Bool
-winGetDDProcAddresses ();
+winGetDDProcAddresses (void);
 
 
 /*
  * winerror.c
  */
 
+#ifdef DDXOSVERRORF
 void
 OSVenderVErrorF (const char *pszFormat, va_list va_args);
+#endif
+
+void
+winMessageBoxF (const char *pszError, UINT uType, ...);
 
 
+#ifdef XWIN_NATIVEGDI
 /*
  * winfillsp.c
  */
@@ -855,8 +846,10 @@ winFillSpansNativeGDI (DrawablePtr	pDrawable,
 		       DDXPointPtr	pPoints,
 		       int		*pWidths,
 		       int		fSorted);
+#endif
 
 
+#ifdef XWIN_NATIVEGDI
 /*
  * winfont.c
  */
@@ -866,42 +859,20 @@ winRealizeFontNativeGDI (ScreenPtr pScreen, FontPtr pFont);
 
 Bool
 winUnrealizeFontNativeGDI (ScreenPtr pScreen, FontPtr pFont);
+#endif
 
 
+#ifdef XWIN_NATIVEGDI
 /*
  * wingc.c
  */
 
 Bool
 winCreateGCNativeGDI (GCPtr pGC);
-
-void
-winChangeGCNativeGDI (GCPtr pGC, unsigned long ulChanges);
-
-void
-winPadPixmapNativdGDI (PixmapPtr pPixmap);
-
-void
-winValidateGCNativeGDI (GCPtr pGC,
-			unsigned long changes,
-			DrawablePtr pDrawable);
-
-void
-winCopyGCNativeGDI (GCPtr pGCsrc, unsigned long ulMask, GCPtr pGCdst);
-
-void
-winDestroyGCNativeGDI (GCPtr pGC);
-
-void
-winChangeClipNativeGDI (GCPtr pGC, int nType, pointer pValue, int nRects);
-
-void
-winDestroyClipNativeGDI (GCPtr pGC);
-
-void
-winCopyClipNativeGDI (GCPtr pGCdst, GCPtr pGCsrc);
+#endif
 
 
+#ifdef XWIN_NATIVEGDI
 /*
  * wingetsp.c
  */
@@ -913,29 +884,23 @@ winGetSpansNativeGDI (DrawablePtr	pDrawable,
 		      int		*pWidths, 
 		      int		nSpans, 
 		      char		*pDst);
+#endif
+
+
+/*
+ * winglobals.c
+ */
+
+void
+winInitializeGlobals (void);
 
 
 /*
  * winkeybd.c
  */
 
-#if WIN_NEW_KEYBOARD_SUPPORT
-winKeyEventsRec
-winTranslateKey (DWORD dwVirtualKey, DWORD dwKeyData);
-#else
 void
 winTranslateKey (WPARAM wParam, LPARAM lParam, int *piScanCode);
-#endif
-
-void
-winGetKeyMappings (KeySymsPtr pKeySyms, CARD8 *pModMap);
-
-void
-winKeybdBell (int iPercent, DeviceIntPtr pDeviceInt,
-	      pointer pCtrl, int iClass);
-
-void
-winKeybdCtrl (DeviceIntPtr pDevice, KeybdCtrl *pCtrl);
 
 int
 winKeybdProc (DeviceIntPtr pDeviceInt, int iState);
@@ -944,63 +909,38 @@ void
 winInitializeModeKeyStates (void);
 
 void
-winStoreModeKeyStates (ScreenPtr pScreen);
-
-void
-winRestoreModeKeyStates (ScreenPtr pScreen);
+winRestoreModeKeyStates (void);
 
 Bool
 winIsFakeCtrl_L (UINT message, WPARAM wParam, LPARAM lParam);
 
 void
-winKeybdReleaseKeys ();
+winKeybdReleaseKeys (void);
 
 void
 winSendKeyEvent (DWORD dwKey, Bool fDown);
 
-#if WIN_NEW_KEYBOARD_SUPPORT
-void
-winProcessKeyEvent (DWORD dwVirtKey, DWORD dwKeyData);
-#endif
-
 
 /*
- * winlayer.c
+ * winkeyhook.c
  */
 
-#if WIN_LAYER_SUPPORT
-
-LayerPtr
-winLayerCreate (ScreenPtr pScreen);
-
-int
-winLayerAdd (WindowPtr pWindow, pointer value);
-
-int
-winLayerRemove (WindowPtr pWindow, pointer value);
-
-#ifdef RANDR
 Bool
-winRandRGetInfo (ScreenPtr pScreen, Rotation *pRotations);
+winInstallKeyboardHookLL ();
 
-Bool
-winRandRSetConfig (ScreenPtr		pScreen,
-		   Rotation		rotateKind,
-		   RRScreenSizePtr	pSize);
-
-Bool
-winRandRInit (ScreenPtr pScreen);
-#endif /* RANDR */
-#endif /* WIN_LAYER_SUPPORT */
+void
+winRemoveKeyboardHookLL ();
 
 
 /*
  * winmisc.c
  */
 
+#ifdef XWIN_NATIVEGDI
 void
 winQueryBestSizeNativeGDI (int class, unsigned short *pWidth,
 			   unsigned short *pHeight, ScreenPtr pScreen);
+#endif
 
 CARD8
 winCountBits (DWORD dw);
@@ -1008,17 +948,16 @@ winCountBits (DWORD dw);
 Bool
 winUpdateFBPointer (ScreenPtr pScreen, void *pbits);
 
+#ifdef XWIN_NATIVEGDI
 BOOL
 winPaintBackground (HWND hwnd, COLORREF colorref);
+#endif
 
 
 /*
  * winmouse.c
  */
 
-void
-winMouseCtrl (DeviceIntPtr pDevice, PtrCtrl *pCtrl);
-
 int
 winMouseProc (DeviceIntPtr pDeviceInt, int iState);
 
@@ -1033,7 +972,7 @@ winMouseButtonsHandle (ScreenPtr pScreen,
 		       int iEventType, int iButton,
 		       WPARAM wParam);
 
-
+#ifdef XWIN_NATIVEGDI
 /*
  * winnativegdi.c
  */
@@ -1042,79 +981,22 @@ HBITMAP
 winCreateDIBNativeGDI (int iWidth, int iHeight, int iDepth,
 		       BYTE **ppbBits, BITMAPINFO **ppbmi);
 
-Bool
-winAllocateFBNativeGDI (ScreenPtr pScreen);
-
-void
-winShadowUpdateNativeGDI (ScreenPtr pScreen, 
-			  shadowBufPtr pBuf);
-
-Bool
-winCloseScreenNativeGDI (int nIndex, ScreenPtr pScreen);
-
-Bool
-winInitVisualsNativeGDI (ScreenPtr pScreen);
-
-Bool
-winAdjustVideoModeNativeGDI (ScreenPtr pScreen);
-
-Bool
-winBltExposedRegionsNativeGDI (ScreenPtr pScreen);
-
-Bool
-winActivateAppNativeGDI (ScreenPtr pScreen);
-
 Bool
 winSetEngineFunctionsNativeGDI (ScreenPtr pScreen);
-
-Bool
-winRedrawScreenNativeGDI (ScreenPtr pScreen);
-
-Bool
-winRealizeInstalledPaletteNativeGDI (ScreenPtr pScreen);
-
-Bool
-winInstallColormapNativeGDI (ColormapPtr pColormap);
-
-Bool
-winStoreColorsNativeGDI (ColormapPtr pmap, 
-			 int ndef,
-			 xColorItem *pdefs);
-
-Bool
-winCreateColormapNativeGDI (ColormapPtr pColormap);
-
-Bool
-winDestroyColormapNativeGDI (ColormapPtr pColormap);
-
+#endif
 
 
+#ifdef XWIN_PRIMARYFB
 /*
  * winpfbddd.c
  */
 
-Bool
-winAllocateFBPrimaryDD (ScreenPtr pScreen);
-
-Bool
-winCloseScreenPrimaryDD (int nIndex, ScreenPtr pScreen);
-
-Bool
-winInitVisualsPrimaryDD (ScreenPtr pScreen);
-
-Bool
-winAdjustVideoModePrimaryDD (ScreenPtr pScreen);
-
-Bool
-winActivateAppPrimaryDD (ScreenPtr pScreen);
-
 Bool
 winSetEngineFunctionsPrimaryDD (ScreenPtr pScreen);
-
-Bool
-winHotKeyAltTabPrimaryDD (ScreenPtr pScreen);
+#endif
 
 
+#ifdef XWIN_NATIVEGDI
 /*
  * winpixmap.c
  */
@@ -1125,16 +1007,6 @@ winCreatePixmapNativeGDI (ScreenPtr pScreen, int width, int height, int depth);
 Bool
 winDestroyPixmapNativeGDI (PixmapPtr pPixmap);
 
-void
-winXRotatePixmapNativeGDI (PixmapPtr pPix, int rw);
-
-void
-winYRotatePixmapNativeGDI (PixmapPtr pPix, int rh);
-
-void
-winCopyRotatePixmapNativeGDI (PixmapPtr psrcPix, PixmapPtr *ppdstPix,
-			      int xrot, int yrot);
-
 Bool
 winModifyPixmapHeaderNativeGDI (PixmapPtr pPixmap,
 				int iWidth, int iHeight,
@@ -1142,16 +1014,20 @@ winModifyPixmapHeaderNativeGDI (PixmapPtr pPixmap,
 				int iBitsPerPixel,
 				int devKind,
 				pointer pPixData);
+#endif
 
 
+#ifdef XWIN_NATIVEGDI
 /*
  * winpntwin.c
  */
 
 void
 winPaintWindowNativeGDI (WindowPtr pWin, RegionPtr pRegion, int what);
+#endif
 
 
+#ifdef XWIN_NATIVEGDI
 /*
  * winpolyline.c
  */
@@ -1162,6 +1038,18 @@ winPolyLineNativeGDI (DrawablePtr	pDrawable,
 		      int		mode,
 		      int		npt,
 		      DDXPointPtr	ppt);
+#endif
+
+
+#ifdef XWIN_NATIVEGDI
+/*
+ * winpushpxl.c
+ */
+
+void
+winPushPixels (GCPtr pGC, PixmapPtr pBitMap, DrawablePtr pDrawable,
+	       int dx, int dy, int xOrg, int yOrg);
+#endif
 
 
 /*
@@ -1178,21 +1066,15 @@ winFinishScreenInitFB (int index,
 		       ScreenPtr pScreen,
 		       int argc, char **argv);
 
+#if defined(XWIN_NATIVEGDI)
 Bool
 winFinishScreenInitNativeGDI (int index,
 			      ScreenPtr pScreen,
 			      int argc, char **argv);
-
-Bool
-winSaveScreen (ScreenPtr pScreen, int on);
-
-PixmapPtr
-winGetWindowPixmap (WindowPtr pWin);
-
-void
-winSetWindowPixmap (WindowPtr pWin, PixmapPtr pPix);
+#endif
 
 
+#ifdef XWIN_NATIVEGDI
 /*
  * winsetsp.c
  */
@@ -1205,165 +1087,32 @@ winSetSpansNativeGDI (DrawablePtr	pDrawable,
 		      int		*pWidth,
 		      int		nSpans,
 		      int		fSorted);
+#endif
+
 
 /*
  * winshaddd.c
  */
 
-Bool
-winAllocateFBShadowDD (ScreenPtr pScreen);
-
-void
-winShadowUpdateDD (ScreenPtr pScreen, 
-		   shadowBufPtr pBuf);
-
-Bool
-winCloseScreenShadowDD (int nIndex, ScreenPtr pScreen);
-
-Bool
-winInitVisualsShadowDD (ScreenPtr pScreen);
-
-Bool
-winAdjustVideoModeShadowDD (ScreenPtr pScreen);
-
-Bool
-winBltExposedRegionsShadowDD (ScreenPtr pScreen);
-
-Bool
-winActivateAppShadowDD (ScreenPtr pScreen);
-
 Bool
 winSetEngineFunctionsShadowDD (ScreenPtr pScreen);
 
-Bool
-winRedrawScreenShadowDD (ScreenPtr pScreen);
-
-Bool
-winRealizeInstalledPaletteShadowDD (ScreenPtr pScreen);
-
-Bool
-winInstallColormapShadowDD (ColormapPtr pColormap);
-
-Bool
-winStoreColorsShadowDD (ColormapPtr pmap, 
-			int ndef,
-			xColorItem *pdefs);
-
-Bool
-winCreateColormapShadowDD (ColormapPtr pColormap);
-
-Bool
-winDestroyColormapShadowDD (ColormapPtr pColormap);
-
-Bool
-winCreatePrimarySurfaceShadowDD (ScreenPtr pScreen);
-
-Bool
-winReleasePrimarySurfaceShadowDD (ScreenPtr pScreen);
-
 
 /*
  * winshadddnl.c
  */
 
-Bool
-winAllocateFBShadowDDNL (ScreenPtr pScreen);
-
-void
-winShadowUpdateDDNL (ScreenPtr pScreen, 
-		     shadowBufPtr pBuf);
-
-Bool
-winCloseScreenShadowDDNL (int nIndex, ScreenPtr pScreen);
-
-Bool
-winInitVisualsShadowDDNL (ScreenPtr pScreen);
-
-Bool
-winAdjustVideoModeShadowDDNL (ScreenPtr pScreen);
-
-Bool
-winBltExposedRegionsShadowDDNL (ScreenPtr pScreen);
-
-Bool
-winActivateAppShadowDDNL (ScreenPtr pScreen);
-
 Bool
 winSetEngineFunctionsShadowDDNL (ScreenPtr pScreen);
 
-Bool
-winRedrawScreenShadowDDNL (ScreenPtr pScreen);
-
-Bool
-winRealizeInstalledPaletteShadowDDNL (ScreenPtr pScreen);
-
-Bool
-winInstallColormapShadowDDNL (ColormapPtr pColormap);
-
-Bool
-winStoreColorsShadowDDNL (ColormapPtr pmap, 
-			  int ndef,
-			  xColorItem *pdefs);
-
-Bool
-winCreateColormapShadowDDNL (ColormapPtr pColormap);
-
-Bool
-winDestroyColormapShadowDDNL (ColormapPtr pColormap);
-
-Bool
-winCreatePrimarySurfaceShadowDDNL (ScreenPtr pScreen);
-
-Bool
-winReleasePrimarySurfaceShadowDDNL (ScreenPtr pScreen);
-
 
 /*
  * winshadgdi.c
  */
 
-Bool
-winAllocateFBShadowGDI (ScreenPtr pScreen);
-
-void
-winShadowUpdateGDI (ScreenPtr pScreen, 
-		    shadowBufPtr pBuf);
-
-Bool
-winCloseScreenShadowGDI (int nIndex, ScreenPtr pScreen);
-
-Bool
-winInitVisualsShadowGDI (ScreenPtr pScreen);
-
-Bool
-winAdjustVideoModeShadowGDI (ScreenPtr pScreen);
-
-Bool
-winActivateAppShadowGDI (ScreenPtr pScreen);
-
-Bool
-winRedrawScreenShadowGDI (ScreenPtr pScreen);
-
 Bool
 winSetEngineFunctionsShadowGDI (ScreenPtr pScreen);
 
-Bool
-winRealizeInstalledPaletteShadowGDI (ScreenPtr pScreen);
-
-Bool
-winInstallColormapShadowGDI (ColormapPtr pColormap);
-
-Bool
-winStoreColorsShadowGDI (ColormapPtr pmap, 
-			 int ndef,
-			 xColorItem *pdefs);
-
-Bool
-winCreateColormapShadowGDI (ColormapPtr pColormap);
-
-Bool
-winDestroyColormapShadowGDI (ColormapPtr pColormap);
-
 
 /*
  * winwakeup.c
@@ -1380,6 +1129,7 @@ winWakeupHandler (int nScreen,
  * winwindow.c
  */
 
+#ifdef XWIN_NATIVEGDI
 Bool
 winCreateWindowNativeGDI (WindowPtr pWin);
 
@@ -1402,47 +1152,46 @@ winUnmapWindowNativeGDI (WindowPtr pWindow);
 
 Bool
 winMapWindowNativeGDI (WindowPtr pWindow);
+#endif
 
 Bool
-winCreateWindowPRootless (WindowPtr pWindow);
+winCreateWindowRootless (WindowPtr pWindow);
 
 Bool
-winDestroyWindowPRootless (WindowPtr pWindow);
+winDestroyWindowRootless (WindowPtr pWindow);
 
 Bool
-winPositionWindowPRootless (WindowPtr pWindow, int x, int y);
+winPositionWindowRootless (WindowPtr pWindow, int x, int y);
 
 Bool
-winChangeWindowAttributesPRootless (WindowPtr pWindow, unsigned long mask);
+winChangeWindowAttributesRootless (WindowPtr pWindow, unsigned long mask);
 
 Bool
-winUnmapWindowPRootless (WindowPtr pWindow);
+winUnmapWindowRootless (WindowPtr pWindow);
 
 Bool
-winMapWindowPRootless (WindowPtr pWindow);
+winMapWindowRootless (WindowPtr pWindow);
 
 #ifdef SHAPE
 void
-winSetShapePRootless (WindowPtr pWindow);
+winSetShapeRootless (WindowPtr pWindow);
 #endif
 
 
 /*
- * winmultiwindowicons.c
+ * winmultiwindowicons.c - Used by both multi-window and Win32Rootless
  */
 
 HICON
-winXIconToHICON (WindowPtr pWin);
-
-void
-winUpdateIcon (Window id);
+winXIconToHICON (WindowPtr pWin, int iconSize);
 
 
+#ifdef XWIN_MULTIWINDOW
 /*
  * winmultiwindowshape.c
  */
 
-#ifdef SHAPE
+# ifdef SHAPE
 void
 winReshapeMultiWindow (WindowPtr pWin);
 
@@ -1451,9 +1200,11 @@ winSetShapeMultiWindow (WindowPtr pWindow);
 
 void
 winUpdateRgnMultiWindow (WindowPtr pWindow);
+# endif
 #endif
 
 
+#ifdef XWIN_MULTIWINDOW
 /*
  * winmultiwindowwindow.c
  */
@@ -1483,18 +1234,28 @@ void
 winRestackWindowMultiWindow (WindowPtr pWin, WindowPtr pOldNextSib);
 
 void
-winReorderWindowsMultiWindow (ScreenPtr pScreen);
+winReorderWindowsMultiWindow (void);
 
 void
-winMoveXWindow (WindowPtr pWin, int x, int y);
+winResizeWindowMultiWindow (WindowPtr pWin, int x, int y, unsigned int w,
+			    unsigned int h, WindowPtr pSib);
+void
+winMoveWindowMultiWindow (WindowPtr pWin, int x, int y,
+			  WindowPtr pSib, VTKind kind);
 
 void
-winResizeXWindow (WindowPtr pWin, int w, int h);
+winCopyWindowMultiWindow (WindowPtr pWin, DDXPointRec oldpt,
+			  RegionPtr oldRegion);
 
 XID
 winGetWindowID (WindowPtr pWin);
 
+int
+winAdjustXWindow (WindowPtr pWin, HWND hwnd);
+#endif
 
+
+#ifdef XWIN_MULTIWINDOW
 /*
  * winmultiwindowwndproc.c
  */
@@ -1502,6 +1263,7 @@ winGetWindowID (WindowPtr pWin);
 LRESULT CALLBACK
 winTopLevelWindowProc (HWND hwnd, UINT message, 
 		       WPARAM wParam, LPARAM lParam);
+#endif
 
 
 /*
@@ -1529,6 +1291,123 @@ winWindowProc (HWND hWnd, UINT message,
 	       WPARAM wParam, LPARAM lParam);
 
 
+#ifdef XWIN_MULTIWINDOWEXTWM
+/*
+ * winwin32rootless.c
+ */
+
+Bool
+winMWExtWMCreateFrame (RootlessWindowPtr pFrame, ScreenPtr pScreen,
+			     int newX, int newY, RegionPtr pShape);
+
+void
+winMWExtWMDestroyFrame (RootlessFrameID wid);
+
+void
+winMWExtWMMoveFrame (RootlessFrameID wid, ScreenPtr pScreen, int newX, int newY);
+
+void
+winMWExtWMResizeFrame (RootlessFrameID wid, ScreenPtr pScreen,
+			     int newX, int newY, unsigned int newW, unsigned int newH,
+			     unsigned int gravity);
+
+void
+winMWExtWMRestackFrame (RootlessFrameID wid, RootlessFrameID nextWid);
+
+void
+winMWExtWMReshapeFrame (RootlessFrameID wid, RegionPtr pShape);
+
+void
+winMWExtWMUnmapFrame (RootlessFrameID wid);
+
+void
+winMWExtWMStartDrawing (RootlessFrameID wid, char **pixelData, int *bytesPerRow);
+
+void
+winMWExtWMStopDrawing (RootlessFrameID wid, Bool flush);
+
+void
+winMWExtWMUpdateRegion (RootlessFrameID wid, RegionPtr pDamage);
+
+void
+winMWExtWMDamageRects (RootlessFrameID wid, int count, const BoxRec *rects,
+			     int shift_x, int shift_y);
+
+void
+winMWExtWMRootlessSwitchWindow (RootlessWindowPtr pFrame, WindowPtr oldWin);
+
+void
+winMWExtWMCopyBytes (unsigned int width, unsigned int height,
+			   const void *src, unsigned int srcRowBytes,
+			   void *dst, unsigned int dstRowBytes);
+
+void
+winMWExtWMFillBytes (unsigned int width, unsigned int height, unsigned int value,
+			   void *dst, unsigned int dstRowBytes);
+
+int
+winMWExtWMCompositePixels (unsigned int width, unsigned int height, unsigned int function,
+				 void *src[2], unsigned int srcRowBytes[2],
+				 void *mask, unsigned int maskRowBytes,
+				 void *dst[2], unsigned int dstRowBytes[2]);
+
+void
+winMWExtWMCopyWindow (RootlessFrameID wid, int dstNrects, const BoxRec *dstRects,
+			    int dx, int dy);
+#endif
+
+
+#ifdef XWIN_MULTIWINDOWEXTWM
+/*
+ * winwin32rootlesswindow.c
+ */
+
+void
+winMWExtWMReorderWindows (ScreenPtr pScreen);
+
+void
+winMWExtWMMoveXWindow (WindowPtr pWin, int x, int y);
+
+void
+winMWExtWMResizeXWindow (WindowPtr pWin, int w, int h);
+
+void
+winMWExtWMMoveResizeXWindow (WindowPtr pWin, int x, int y, int w, int h);
+
+void
+winMWExtWMUpdateIcon (Window id);
+#endif
+
+
+#ifdef XWIN_MULTIWINDOWEXTWM
+/*
+ * winwin32rootlesswndproc.c
+ */
+
+LRESULT CALLBACK
+winMWExtWMWindowProc (HWND hwnd, UINT message, 
+			    WPARAM wParam, LPARAM lParam);
+#endif
+
+
+/*
+ * winwindowswm.c
+ */
+
+void
+winWindowsWMSendEvent (int type, unsigned int mask, int which, int arg,
+		       Window window, int x, int y, int w, int h);
+
+void
+winWindowsWMExtensionInit (void);
+
+/*
+ * wincursor.c
+ */
+
+Bool
+winInitCursor (ScreenPtr pScreen);
+
 /*
  * END DDX and DIX Function Prototypes
  */
